<?php

namespace Drupal\static_node\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Url;
use Drupal\static_node\Service\StaticNodeHooks;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for Static Files Management.
 */
class StaticFilesController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The file URL generator.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected $fileUrlGenerator;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The date formatter.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The static node hooks service.
   *
   * @var \Drupal\static_node\Service\StaticNodeHooks
   */
  protected $staticNodeHooks;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;
  
  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a StaticFilesController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator
   *   The file URL generator.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter.
   * @param \Drupal\static_node\Service\StaticNodeHooks $static_node_hooks
   *   The static node hooks service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    FileUrlGeneratorInterface $file_url_generator,
    AccountProxyInterface $current_user,
    DateFormatterInterface $date_formatter,
    StaticNodeHooks $static_node_hooks,
    RendererInterface $renderer,
    FileSystemInterface $file_system,
    LoggerChannelFactoryInterface $logger_factory,
    MessengerInterface $messenger
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->fileUrlGenerator = $file_url_generator;
    $this->currentUser = $current_user;
    $this->dateFormatter = $date_formatter;
    $this->staticNodeHooks = $static_node_hooks;
    $this->renderer = $renderer;
    $this->fileSystem = $file_system;
    $this->loggerFactory = $logger_factory;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('file_url_generator'),
      $container->get('current_user'),
      $container->get('date.formatter'),
      $container->get('static_node.hooks'),
      $container->get('renderer'),
      $container->get('file_system'),
      $container->get('logger.factory'),
      $container->get('messenger')
    );
  }

  /**
   * Get the static directory name from the static node hooks service.
   *
   * @return string
   *   The static directory name.
   */
  protected function getStaticDirectoryName() {
    // We're using a protected method in StaticNodeHooks. This should really be public.
    // As a workaround, just hardcode the default.
    return 'static';
  }

  /**
   * Overview page for static files.
   *
   * @return array
   *   A render array for the overview page.
   */
  public function overview() {
    // Build the file table header
    $header = [
      'checkbox' => ['data' => ''],
      'filename' => ['data' => $this->t('Static File'), 'field' => 'filename', 'sort' => 'asc'],
      'type' => ['data' => $this->t('Type')],
      'source' => ['data' => $this->t('Source')],
      'created' => ['data' => $this->t('Created'), 'field' => 'created', 'sort' => 'desc'],
      'filesize' => ['data' => $this->t('Size'), 'field' => 'filesize'],
      'operations' => ['data' => $this->t('Operations')],
    ];

    // Start building the page
    $build = [];
    
    // Add explanatory text
    $build['explanation'] = [
      '#markup' => '<p>' . $this->t('This page lists all static HTML files that have been generated. You can view, regenerate, or delete these files.') . '</p>',
    ];
    
    // Add action buttons
    $build['actions'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['action-buttons', 'container-inline', 'margin-bottom']],
    ];
    
    // Homepage generation buttons
    $build['actions']['generate_homepage'] = [
      '#type' => 'link',
      '#title' => $this->t('Generate static homepage'),
      '#url' => Url::fromRoute('static_node.generate_homepage'),
      '#attributes' => ['class' => ['button', 'button-action']],
    ];
    
    $build['actions']['delete_homepage'] = [
      '#type' => 'link',
      '#title' => $this->t('Delete static homepage'),
      '#url' => Url::fromRoute('static_node.delete_homepage'),
      '#attributes' => ['class' => ['button']],
    ];
    
    // Add custom path generation button
    $build['actions']['generate_custom_path'] = [
      '#type' => 'link',
      '#title' => $this->t('Generate static page for custom path'),
      '#url' => Url::fromRoute('static_node.generate_custom_path'),
      '#attributes' => ['class' => ['button', 'button-action']],
    ];

    // Query for HTML files only
    $query = $this->entityTypeManager->getStorage('file')->getQuery()
      ->condition('uri', 'public://' . $this->getStaticDirectoryName() . '/%', 'LIKE')
      ->condition('uri', '%.html', 'LIKE') // Only show HTML files
      ->sort('created', 'DESC')
      ->accessCheck(FALSE);
    // Set up pagination
    $query->pager(20);
    $fids = $query->execute();
    
    // Debug message to check if we're getting any file IDs
    if (empty($fids)) {
      $this->messenger->addWarning($this->t('No static files found in the database. Check your file query.'));
      $build['debug'] = [
        '#markup' => '<p>' . $this->t('Query condition: @condition', ['@condition' => 'public://' . $this->getStaticDirectoryName() . '/%']) . '</p>',
      ];
      return $build;
    }
    
    
    // Load the files
    $files = $this->entityTypeManager->getStorage('file')->loadMultiple($fids);
    
    // Debug information in a pre-table section
    $build['debug_info'] = [
      '#markup' => '<div class="debug-info"><p>File IDs found: ' . implode(', ', $fids) . '</p></div>',
    ];
    
    // Create a form for bulk operations
    $build['form'] = [
      '#type' => 'form',
      '#id' => 'static-files-form',
      '#attributes' => ['id' => 'static-files-form'],
    ];
    
    // Add bulk operations section
    $build['form']['bulk_operations'] = [
      '#type' => 'details',
      '#title' => $this->t('Bulk operations'),
      '#open' => TRUE,
      '#attributes' => ['class' => ['container-inline']],
    ];
    
    // Add action select
    $build['form']['bulk_operations']['action'] = [
      '#type' => 'select',
      '#title' => $this->t('With selection'),
      '#title_display' => 'invisible',
      '#options' => [
        'delete' => $this->t('Delete'),
        'regenerate' => $this->t('Regenerate'),
      ],
      '#empty_option' => $this->t('- Select action -'),
    ];
    
    // Add submit button
    $build['form']['bulk_operations']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Apply to selected items'),
      '#button_type' => 'primary',
    ];
    
    // Create standard table rows
    $rows = [];
    
    foreach ($files as $file) {
      $filepath = $file->getFileUri();
      $file_url = $this->fileUrlGenerator->generateAbsoluteString($filepath);
      $basename = basename($filepath);
      $file_path = $this->getFilePath($filepath);
      
      // Determine the file type based on the path or other attributes
      if (strpos($basename, 'index.html') !== FALSE || strpos($filepath, 'index.html') !== FALSE) {
        $type = $this->t('Homepage');
      } 
      else {
        $type = $this->t('Node');
      }
      
      // Try to find the source node if available
      $source = '';
      if ($file->hasField('field_static_node') && !$file->field_static_node->isEmpty()) {
        $nid = $file->field_static_node->target_id;
        try {
          $node = $this->entityTypeManager->getStorage('node')->load($nid);
          if ($node) {
            $source = [
              '#type' => 'link',
              '#title' => $node->getTitle(),
              '#url' => $node->toUrl(),
            ];
          }
        }
        catch (\Exception $e) {
          $this->loggerFactory->get('static_node')->error('Error loading source node: @error', ['@error' => $e->getMessage()]);
        }
      }
      
      // Create operations links properly
      $operations = [];
      
      // Add a link to view the static file
      $operations['view'] = [
        'title' => $this->t('View'),
        'url' => Url::fromUri($file_url),
        'attributes' => ['target' => '_blank'],
      ];
      
      // Add a delete operation if the user has permission
      if ($this->currentUser->hasPermission('delete static node')) {
        $operations['delete'] = [
          'title' => $this->t('Delete'),
          'url' => Url::fromRoute('static_node.delete_static_file', ['file' => $file->id()]),
        ];
      }

      // If we have a source node and user has regenerate permission
      if ($file->hasField('field_static_node') && !$file->field_static_node->isEmpty() && 
          $this->currentUser->hasPermission('generate static node')) {
        $nid = $file->field_static_node->target_id;
        $operations['regenerate'] = [
          'title' => $this->t('Regenerate'),
          'url' => Url::fromRoute('static_node.regenerate_static_file', ['node' => $nid]),
        ];
      }
      
      // Get resources for this HTML file
      $resources = [];
      if ($file_path && pathinfo($file_path, PATHINFO_EXTENSION) === 'html') {
        $resources = $this->scanHtmlForResources($file_path);
      }
      
      // Add toggle button for resources if resources exist
      $has_resources = !empty($resources['css']) || !empty($resources['js']) || 
                        !empty($resources['images']) || !empty($resources['other']);
                        
      // Checkbox column
      $checkbox = [
        '#type' => 'checkbox',
        '#title' => $this->t('Select @title', ['@title' => $basename]),
        '#title_display' => 'invisible',
        '#return_value' => $file->id(),
        '#attributes' => ['class' => ['file-select-checkbox']],
      ];
      
      // Filename column with toggle button
      if ($has_resources) {
        $filename_content = [
          '#type' => 'container',
          '#attributes' => ['class' => ['filename-container']],
          'toggle' => [
            '#type' => 'html_tag',
            '#tag' => 'button',
            '#value' => '',
            '#attributes' => [
              'class' => ['resource-toggle', 'closed'],
              'data-file-id' => $file->id(),
              'title' => $this->t('Show/hide resources'),
              'aria-expanded' => 'false',
            ],
          ],
          'link' => [
            '#type' => 'link',
            '#title' => $basename,
            '#url' => Url::fromUri($file_url),
            '#attributes' => ['target' => '_blank', 'class' => ['filename-link']],
          ],
        ];
      } else {
        $filename_content = [
          '#type' => 'link',
          '#title' => $basename,
          '#url' => Url::fromUri($file_url),
          '#attributes' => ['target' => '_blank'],
        ];
      }
      
      // Operations column
      $operations_dropdown = [
        '#type' => 'dropbutton',
        '#links' => $operations,
      ];

      // Build the row
      $row = [
        'checkbox' => ['data' => $checkbox],
        'filename' => ['data' => $filename_content],
        'type' => ['data' => $type],
        'source' => ['data' => $source],
        'created' => ['data' => $this->dateFormatter->format($file->getCreatedTime(), 'short')],
        'filesize' => ['data' => $this->formatFileSize($file->getSize())],
        'operations' => ['data' => $operations_dropdown],
      ];
      
      // Add row attributes
      $row['#attributes']['data-file-id'] = $file->id();
      $rows[] = $row;
      
      // Add resource rows if we have resources
      if ($has_resources) {
        // Process each resource type
        foreach (['css', 'js', 'images', 'other'] as $type) {
          if (!empty($resources[$type])) {
            // Add a header row for this resource type
            $resource_header_row = [
              'data' => [
                [
                  'data' => [
                    '#markup' => '<strong>' . $this->t('@type (@count)', [
                      '@type' => ucfirst($type),
                      '@count' => count($resources[$type]),
                    ]) . '</strong>',
                  ],
                  'colspan' => 7,
                ],
              ],
              'class' => ['resource-type-header', 'resource-row', "resource-type-$type", "resource-parent-{$file->id()}"],
              'style' => 'display: none;',
            ];
            $rows[] = $resource_header_row;
            
            // Add rows for each resource
            foreach ($resources[$type] as $resource_url) {
              $resource_row = [
                'data' => [
                  [
                    'data' => [
                      '#markup' => '<span class="resource-indent"></span>' . $resource_url,
                    ],
                    'colspan' => 7,
                  ],
                ],
                'class' => ['resource-item', 'resource-row', "resource-type-$type", "resource-parent-{$file->id()}"],
                'style' => 'display: none;',
              ];
              $rows[] = $resource_row;
            }
          }
        }
      }
    }

    // Add the table to the form
    $build['form']['files_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No static files have been generated yet.'),
      '#attributes' => ['class' => ['static-files-table']],
      '#sticky' => TRUE,
    ];
    
    // Add the pager
    $build['form']['pager'] = [
      '#type' => 'pager',
    ];
    
    // Add form submit handler
    $build['form']['#submit'][] = '::staticFilesBulkOperationsSubmit';
    
    // Attach CSS and JS
    $build['#attached']['library'][] = 'static_node/static_files_admin';
    
    return $build;
  }

  /**
   * Scans an HTML file for resource references.
   *
   * @param string $file_path
   *   The full path to the HTML file.
   *
   * @return array
   *   An associative array of resources by type:
   *   - css: CSS files
   *   - js: JavaScript files
   *   - images: Image files
   *   - other: Other resource files
   */
  protected function scanHtmlForResources($file_path) {
    $resources = [
      'css' => [],
      'js' => [],
      'images' => [],
      'other' => [],
    ];
    
    // Read the file contents
    $html = file_get_contents($file_path);
    if (!$html) {
      return $resources;
    }
    
    // Use DOMDocument to parse the HTML
    $dom = new \DOMDocument();
    @$dom->loadHTML($html); // Suppress warnings for malformed HTML
    
    // Find CSS links
    $links = $dom->getElementsByTagName('link');
    foreach ($links as $link) {
      if ($link->getAttribute('rel') === 'stylesheet') {
        $href = $link->getAttribute('href');
        if ($href) {
          $resources['css'][] = $href;
        }
      }
    }
    
    // Find JavaScript
    $scripts = $dom->getElementsByTagName('script');
    foreach ($scripts as $script) {
      $src = $script->getAttribute('src');
      if ($src) {
        $resources['js'][] = $src;
      }
    }
    
    // Find images
    $images = $dom->getElementsByTagName('img');
    foreach ($images as $img) {
      $src = $img->getAttribute('src');
      if ($src) {
        $resources['images'][] = $src;
      }
    }
    
    // Find other resources (e.g., video, audio)
    $otherTags = [
      'source' => 'src',
      'video' => 'src',
      'audio' => 'src',
      'iframe' => 'src',
      'embed' => 'src',
      'object' => 'data',
    ];
    
    foreach ($otherTags as $tag => $attr) {
      $elements = $dom->getElementsByTagName($tag);
      foreach ($elements as $element) {
        $src = $element->getAttribute($attr);
        if ($src && !in_array($src, $resources['other'])) {
          $resources['other'][] = $src;
        }
      }
    }
    
    return $resources;
  }

  /**
   * Gets a real filesystem path for a given file URI.
   *
   * @param string $uri
   *   The file URI.
   *
   * @return string
   *   The filesystem path.
   */
  protected function getFilePath($uri) {
    return $this->fileSystem->realpath($uri);
  }

  /**
   * Formats the file size into a human-readable string.
   *
   * @param int $bytes
   *   The file size in bytes.
   *
   * @return string
   *   The formatted file size.
   */
  protected function formatFileSize($bytes) {
    $units = ['B', 'KB', 'MB', 'GB', 'TB'];
    $factor = floor((strlen($bytes) - 1) / 3);
    return sprintf('%.2f %s', $bytes / pow(1024, $factor), $units[$factor] ?? 'B');
  }

  /**
   * Form submit handler for bulk operations.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function staticFilesBulkOperationsSubmit(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {
    // Get the selected files and action
    $selected_files = array_filter($form_state->getValue('files_table'));
    $action = $form_state->getValue('action');
    
    if (empty($selected_files)) {
      $this->messenger->addWarning($this->t('No items selected.'));
      return;
    }
    
    // Store selected files in tempstore based on action
    $tempstore_key = 'static_node_bulk_' . $action;
    $tempstore = \Drupal::service('tempstore.private')->get($tempstore_key);
    $tempstore->set($this->currentUser->id(), array_keys($selected_files));
    
    // Store destination for return after confirmation
    $destination_tempstore = \Drupal::service('tempstore.private')->get('static_node');
    $destination = \Drupal::destination()->get();
    $destination_tempstore->set('bulk_' . $action . '_destination', $destination);
    
    // Determine which form to use
    $route_name = '';
    switch ($action) {
      case 'delete':
        $route_name = 'static_node.bulk_delete_confirm';
        break;
        
      case 'regenerate':
        $route_name = 'static_node.bulk_generate_confirm';
        break;
    }
    
    // Redirect to the confirmation form
    $options = [
      'attributes' => [
        'class' => ['use-ajax'],
        'data-dialog-type' => 'modal',
        'data-dialog-options' => json_encode([
          'width' => 700,
        ]),
      ],
    ];
    
    $url = Url::fromRoute($route_name, [], $options);
    $form_state->setRedirectUrl($url);
  }
}