<?php

namespace Drupal\static_node\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Url;
use Drupal\static_node\Service\StaticNodeHooks;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Controller for generating static files for custom paths.
 */
class StaticPageController extends ControllerBase {
  use StringTranslationTrait;

  /**
   * The static node hooks service.
   *
   * @var \Drupal\static_node\Service\StaticNodeHooks
   */
  protected $staticNodeHooks;

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * Constructs a StaticPageController object.
   *
   * @param \Drupal\static_node\Service\StaticNodeHooks $static_node_hooks
   *   The static node hooks service.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder.
   */
  public function __construct(
    StaticNodeHooks $static_node_hooks,
    FormBuilderInterface $form_builder
  ) {
    $this->staticNodeHooks = $static_node_hooks;
    $this->formBuilder = $form_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('static_node.hooks'),
      $container->get('form_builder')
    );
  }

  /**
   * Displays a form for generating static files for custom paths.
   *
   * @return array
   *   A form array.
   */
  public function customPathForm() {
    return $this->formBuilder->getForm('Drupal\static_node\Form\StaticCustomPathForm');
  }

  /**
   * Generates a static file for a specific path.
   *
   * @param string $path
   *   The path to generate a static file for.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect to the files management page.
   */
  public function generateStaticPage($path) {
    // Check access
    if (!$this->currentUser()->hasPermission('generate static node')) {
      $this->messenger()->addError($this->t('You do not have permission to generate static pages.'));
      return new RedirectResponse(Url::fromRoute('static_node.files_management')->toString());
    }

    // Normalize the path (remove leading slash if present)
    $path = ltrim($path, '/');

    // Generate the static page
    $this->staticNodeHooks->generateStaticPageForPath($path);

    // Redirect to the files management page
    return new RedirectResponse(Url::fromRoute('static_node.files_management')->toString());
  }

}