<?php

namespace Drupal\static_node\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\static_node\Service\StaticNodeHooks;

/**
 * Provides a confirmation form for bulk deleting static pages.
 */
class BulkDeleteStaticConfirmForm extends ConfirmFormBase {

  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The static node service.
   *
   * @var \Drupal\static_node\Service\StaticNodeHooks
   */
  protected $staticNodeService;

  /**
   * The selected nodes.
   *
   * @var \Drupal\node\NodeInterface[]
   */
  protected $nodes = [];

  /**
   * Constructs a BulkDeleteStaticConfirmForm object.
   *
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The tempstore factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\static_node\Service\StaticNodeHooks $static_node_service
   *   The static node service.
   */
  public function __construct(
    PrivateTempStoreFactory $temp_store_factory,
    EntityTypeManagerInterface $entity_type_manager,
    AccountInterface $current_user,
    StaticNodeHooks $static_node_service
  ) {
    $this->tempStoreFactory = $temp_store_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
    $this->staticNodeService = $static_node_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('tempstore.private'),
      $container->get('entity_type.manager'),
      $container->get('current_user'),
      $container->get('static_node.hooks')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_node_bulk_delete_confirm_form';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->formatPlural(
      count($this->nodes),
      'Are you sure you want to delete the static page for this content item?',
      'Are you sure you want to delete static pages for these @count content items?'
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    // Check if there's a stored destination from the bulk action
    $tempstore = $this->tempStoreFactory->get('static_node');
    $destination = $tempstore->get('bulk_delete_destination');
    
    if ($destination) {
      return Url::fromUserInput('/' . $destination);
    }
    
    // Default to admin content page if no destination stored
    return new Url('system.admin');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->t('This will delete the static HTML versions of these pages. The original content will not be affected, but anonymous users will see the dynamic version of the pages instead.');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Delete Static Pages');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Load node IDs from the tempstore.
    $this->nodes = [];
    $node_ids = $this->tempStoreFactory
      ->get('static_node_bulk_delete')
      ->get($this->currentUser->id());
    
    if (!$node_ids) {
      return $this->redirect('system.admin');
    }
    
    $this->nodes = $this->entityTypeManager->getStorage('node')->loadMultiple($node_ids);
    
    $items = [];
    foreach ($this->nodes as $node) {
      $items[] = $node->label();
    }
    
    $form['nodes'] = [
      '#theme' => 'item_list',
      '#title' => $this->t('The following content items will have static pages deleted:'),
      '#items' => $items,
    ];
    
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    if (empty($this->nodes)) {
      $this->messenger()->addWarning($this->t('No content items were selected for static page deletion.'));
      return;
    }
    
    // Process each node.
    $temp_form = [];
    $processed = 0;
    
    foreach ($this->nodes as $node) {
      // Create a new form state and set the node
      $node_form_state = new \Drupal\Core\Form\FormState();
      $node_form_state->set('node', $node);
      
      // Delete the static version using the service
      $this->staticNodeService->deleteStaticVersion($temp_form, $node_form_state);
      
      $processed++;
    }
    
    // Report the results.
    if ($processed > 0) {
      $this->messenger()->addStatus($this->formatPlural(
        $processed,
        'Deleted 1 static page successfully.',
        'Deleted @count static pages successfully.'
      ));
    }
    
    // Delete the tempstore.
    $this->tempStoreFactory
      ->get('static_node_bulk_delete')
      ->delete($this->currentUser->id());
    
    // Get stored destination
    $tempstore = $this->tempStoreFactory->get('static_node');
    $destination = $tempstore->get('bulk_delete_destination');
    
    // Clean up the stored destination
    if ($destination) {
      $tempstore->delete('bulk_delete_destination');
      $form_state->setRedirectUrl(Url::fromUserInput('/' . $destination));
    }
    else {
      // Check for the destination query parameter as fallback
      $request = \Drupal::requestStack()->getCurrentRequest();
      if ($request->query->has('destination')) {
        $destination = $request->query->get('destination');
        $request->query->remove('destination');
        $form_state->setRedirectUrl(Url::fromUserInput('/' . $destination));
      }
      else {
        $form_state->setRedirectUrl(new Url('system.admin'));
      }
    }
  }
}
