<?php

namespace Drupal\static_node\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\static_node\Service\StaticNodeHooks;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form for generating static files for custom paths.
 */
class StaticCustomPathForm extends FormBase {

  /**
   * The static node hooks service.
   *
   * @var \Drupal\static_node\Service\StaticNodeHooks
   */
  protected $staticNodeHooks;

  /**
   * Constructs a StaticCustomPathForm object.
   *
   * @param \Drupal\static_node\Service\StaticNodeHooks $static_node_hooks
   *   The static node hooks service.
   */
  public function __construct(StaticNodeHooks $static_node_hooks) {
    $this->staticNodeHooks = $static_node_hooks;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('static_node.hooks')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_node_custom_path_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['description'] = [
      '#markup' => '<p>' . $this->t('Use this form to generate a static HTML file for a specific path, such as a Views page, taxonomy term page, or any other page that is not a node.') . '</p>',
    ];

    $form['path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Path'),
      '#description' => $this->t('Enter the path for which you want to generate a static file. For example: "news" or "/about-us".'),
      '#required' => TRUE,
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Generate Static File'),
      '#button_type' => 'primary',
    ];

    $form['actions']['cancel'] = [
      '#type' => 'link',
      '#title' => $this->t('Cancel'),
      '#url' => $this->redirect('static_node.files_management'),
      '#attributes' => ['class' => ['button']],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $path = $form_state->getValue('path');
    
    // Ensure the path doesn't start with 'http' (full URLs are not supported)
    if (strpos($path, 'http') === 0) {
      $form_state->setErrorByName('path', $this->t('Please enter a relative path without the domain.'));
    }

    // Ensure the path doesn't include query parameters
    if (strpos($path, '?') !== FALSE) {
      $form_state->setErrorByName('path', $this->t('Paths with query parameters are not supported.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $path = $form_state->getValue('path');
    
    // Normalize the path (ensure it starts with a slash)
    $path = '/' . ltrim($path, '/');
    
    // Generate the static page
    $this->staticNodeHooks->generateStaticPageForPath($path);
    
    // Redirect to the files management page
    $form_state->setRedirect('static_node.files_management');
  }

}