<?php

namespace Drupal\static_node\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\node\NodeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\static_node\Service\StaticNodeHooks;

/**
 * Form for confirming static file deletion.
 */
class StaticFileDeleteForm extends ConfirmFormBase {

  /**
   * The node for which to delete a static file.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected $node;

  /**
   * The static node service.
   *
   * @var \Drupal\static_node\Service\StaticNodeHooks
   */
  protected $staticNodeService;

  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * Constructs a new StaticFileDeleteForm.
   *
   * @param \Drupal\static_node\Service\StaticNodeHooks $static_node_service
   *   The static node service.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The tempstore factory.
   */
  public function __construct(StaticNodeHooks $static_node_service, PrivateTempStoreFactory $temp_store_factory) {
    $this->staticNodeService = $static_node_service;
    $this->tempStoreFactory = $temp_store_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('static_node.hooks'),
      $container->get('tempstore.private')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_node_delete_confirm_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, NodeInterface $node = NULL) {
    $this->node = $node;
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to delete the static HTML file for %title?', ['%title' => $this->node->label()]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('entity.node.canonical', ['node' => $this->node->id()]);
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->t('This will remove the static HTML version of this page, and anonymous users will see the dynamic version instead. This action can be undone by regenerating the static file later.');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Delete Static File');
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Create an empty form array
    $temp_form = [];
    // Use the existing form state and set the node
    $form_state->set('node', $this->node);
    
    // Delete the static version using the service
    $this->staticNodeService->deleteStaticVersion($temp_form, $form_state);
    
    // Check if we have a stored original destination to return to
    $tempstore = $this->tempStoreFactory->get('static_node');
    $original_destination = $tempstore->get('original_destination_' . $this->node->id());
    
    if ($original_destination) {
      // Clear the stored value as we've used it
      $tempstore->delete('original_destination_' . $this->node->id());
      $form_state->setRedirectUrl(Url::fromUserInput('/' . $original_destination));
    }
    else {
      // Default to returning to the node
      $form_state->setRedirectUrl(new Url('entity.node.canonical', ['node' => $this->node->id()]));
    }
  }
}
