<?php

namespace Drupal\static_node\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\static_node\Service\StaticNodeHooks;

/**
 * Provides a confirmation form for deleting multiple static files.
 */
class StaticFilesBulkDeleteConfirmForm extends ConfirmFormBase {

  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The static node service.
   *
   * @var \Drupal\static_node\Service\StaticNodeHooks
   */
  protected $staticNodeService;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The static files to be deleted.
   *
   * @var \Drupal\file\FileInterface[]
   */
  protected $files = [];

  /**
   * Constructs a StaticFilesBulkDeleteConfirmForm object.
   *
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The tempstore factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\static_node\Service\StaticNodeHooks $static_node_service
   *   The static node service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(
    PrivateTempStoreFactory $temp_store_factory,
    EntityTypeManagerInterface $entity_type_manager,
    AccountInterface $current_user,
    StaticNodeHooks $static_node_service,
    MessengerInterface $messenger,
    LoggerChannelFactoryInterface $logger_factory
  ) {
    $this->tempStoreFactory = $temp_store_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
    $this->staticNodeService = $static_node_service;
    $this->messenger = $messenger;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('tempstore.private'),
      $container->get('entity_type.manager'),
      $container->get('current_user'),
      $container->get('static_node.hooks'),
      $container->get('messenger'),
      $container->get('logger.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_files_bulk_delete_confirm';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    $count = count($this->files);
    return $this->formatPlural($count, 'Are you sure you want to delete this static file?', 'Are you sure you want to delete these @count static files?');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    // Get the stored destination or default to the static files overview.
    $destination_tempstore = $this->tempStoreFactory->get('static_node');
    $destination = $destination_tempstore->get('bulk_delete_destination') ?: '';
    
    if ($destination) {
      return Url::fromUri('internal:' . $destination);
    }
    
    return Url::fromRoute('static_node.files_overview');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->t('This action cannot be undone. The files will be permanently removed.');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Delete files');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Load the selected files from the tempstore.
    $tempstore = $this->tempStoreFactory->get('static_node_bulk_delete');
    $file_ids = $tempstore->get($this->currentUser->id());
    
    if (!$file_ids) {
      $this->messenger->addError($this->t('No files were selected for bulk deletion.'));
      return $this->redirect('static_node.files_overview');
    }
    
    $this->files = $this->entityTypeManager->getStorage('file')->loadMultiple($file_ids);
    
    if (empty($this->files)) {
      $this->messenger->addError($this->t('No files were found for deletion.'));
      return $this->redirect('static_node.files_overview');
    }
    
    $items = [];
    foreach ($this->files as $file) {
      $items[] = [
        '#markup' => $file->getFilename(),
      ];
    }
    
    $form['files'] = [
      '#theme' => 'item_list',
      '#items' => $items,
      '#title' => $this->formatPlural(count($this->files), '1 file selected for deletion:', '@count files selected for deletion:'),
    ];
    
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Delete the selected files.
    $file_storage = $this->entityTypeManager->getStorage('file');
    $success_count = 0;
    $error_count = 0;
    
    foreach ($this->files as $file) {
      try {
        // Delete the file and remove the entity.
        $file_storage->delete([$file]);
        $success_count++;
      }
      catch (\Exception $e) {
        $this->loggerFactory->get('static_node')->error('Failed to delete static file @fid: @error', [
          '@fid' => $file->id(),
          '@error' => $e->getMessage(),
        ]);
        $error_count++;
      }
    }
    
    // Show success/failure messages.
    if ($success_count) {
      $this->messenger->addStatus($this->formatPlural(
        $success_count,
        'Deleted 1 static file successfully.',
        'Deleted @count static files successfully.'
      ));
    }
    
    if ($error_count) {
      $this->messenger->addError($this->formatPlural(
        $error_count,
        'Failed to delete 1 static file.',
        'Failed to delete @count static files.'
      ));
    }
    
    // Clean up the tempstore.
    $tempstore = $this->tempStoreFactory->get('static_node_bulk_delete');
    $tempstore->delete($this->currentUser->id());
    
    // Redirect to the overview page.
    $form_state->setRedirect('static_node.files_overview');
  }
}