<?php

namespace Drupal\static_node\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\static_node\Service\StaticNodeHooks;

/**
 * Provides a confirmation form for generating multiple static files.
 */
class StaticFilesBulkGenerateConfirmForm extends ConfirmFormBase {

  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The static node service.
   *
   * @var \Drupal\static_node\Service\StaticNodeHooks
   */
  protected $staticNodeService;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The static node IDs to be generated.
   *
   * @var array
   */
  protected $nodeIds = [];

  /**
   * Constructs a StaticFilesBulkGenerateConfirmForm object.
   *
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The tempstore factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\static_node\Service\StaticNodeHooks $static_node_service
   *   The static node service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(
    PrivateTempStoreFactory $temp_store_factory,
    EntityTypeManagerInterface $entity_type_manager,
    AccountInterface $current_user,
    StaticNodeHooks $static_node_service,
    MessengerInterface $messenger,
    LoggerChannelFactoryInterface $logger_factory
  ) {
    $this->tempStoreFactory = $temp_store_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
    $this->staticNodeService = $static_node_service;
    $this->messenger = $messenger;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('tempstore.private'),
      $container->get('entity_type.manager'),
      $container->get('current_user'),
      $container->get('static_node.hooks'),
      $container->get('messenger'),
      $container->get('logger.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_files_bulk_generate_confirm';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    $count = count($this->nodeIds);
    return $this->formatPlural($count, 'Are you sure you want to generate static file for this node?', 'Are you sure you want to generate static files for these @count nodes?');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    // Get the stored destination or default to the static files overview.
    $destination_tempstore = $this->tempStoreFactory->get('static_node');
    $destination = $destination_tempstore->get('bulk_generate_destination') ?: '';
    
    if ($destination) {
      return Url::fromUri('internal:' . $destination);
    }
    
    return Url::fromRoute('static_node.files_overview');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->t('This will generate static HTML files for the selected nodes.');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Generate files');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Load the selected node IDs from the tempstore.
    $tempstore = $this->tempStoreFactory->get('static_node_bulk_generate');
    $this->nodeIds = $tempstore->get($this->currentUser->id());
    
    if (empty($this->nodeIds)) {
      $this->messenger->addError($this->t('No nodes were selected for static file generation.'));
      return $this->redirect('static_node.files_overview');
    }
    
    // Load the node titles for display.
    $nodes = $this->entityTypeManager->getStorage('node')->loadMultiple($this->nodeIds);
    
    if (empty($nodes)) {
      $this->messenger->addError($this->t('No nodes were found for static file generation.'));
      return $this->redirect('static_node.files_overview');
    }
    
    $items = [];
    foreach ($nodes as $node) {
      $items[] = [
        '#markup' => $node->label(),
      ];
    }
    
    $form['nodes'] = [
      '#theme' => 'item_list',
      '#items' => $items,
      '#title' => $this->formatPlural(count($nodes), '1 node selected for static file generation:', '@count nodes selected for static file generation:'),
    ];
    
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Load the nodes.
    $nodes = $this->entityTypeManager->getStorage('node')->loadMultiple($this->nodeIds);
    $success_count = 0;
    $error_count = 0;
    
    // Generate static files for each node.
    foreach ($nodes as $node) {
      try {
        // Call the static node service to generate the static file.
        $result = $this->staticNodeService->saveStaticFile($node);
        
        if ($result) {
          $success_count++;
        }
        else {
          $error_count++;
        }
      }
      catch (\Exception $e) {
        $this->loggerFactory->get('static_node')->error('Failed to generate static file for node @nid: @error', [
          '@nid' => $node->id(),
          '@error' => $e->getMessage(),
        ]);
        $error_count++;
      }
    }
    
    // Show success/failure messages.
    if ($success_count) {
      $this->messenger->addStatus($this->formatPlural(
        $success_count,
        'Generated 1 static file successfully.',
        'Generated @count static files successfully.'
      ));
    }
    
    if ($error_count) {
      $this->messenger->addError($this->formatPlural(
        $error_count,
        'Failed to generate 1 static file.',
        'Failed to generate @count static files.'
      ));
    }
    
    // Clean up the tempstore.
    $tempstore = $this->tempStoreFactory->get('static_node_bulk_generate');
    $tempstore->delete($this->currentUser->id());
    
    // Redirect to the overview page.
    $form_state->setRedirect('static_node.files_overview');
  }
}