<?php

namespace Drupal\static_node\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;

/**
 * Form handler for static files bulk operations.
 */
class StaticFilesBulkOperationsForm extends FormBase {

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * Constructs a new StaticFilesBulkOperationsForm.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The tempstore factory.
   */
  public function __construct(
    MessengerInterface $messenger,
    AccountProxyInterface $current_user,
    PrivateTempStoreFactory $temp_store_factory
  ) {
    $this->messenger = $messenger;
    $this->currentUser = $current_user;
    $this->tempStoreFactory = $temp_store_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('messenger'),
      $container->get('current_user'),
      $container->get('tempstore.private')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_node_bulk_operations_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#id'] = 'static-files-bulk-form';
    $form['#attributes']['class'][] = 'clearfix';
    
    $form['bulk_operations'] = [
      '#type' => 'details',
      '#title' => $this->t('Bulk operations'),
      '#open' => TRUE,
      '#attributes' => ['class' => ['container-inline']],
    ];
    
    $form['bulk_operations']['action'] = [
      '#type' => 'select',
      '#title' => $this->t('With selection'),
      '#title_display' => 'invisible',
      '#options' => [
        'delete' => $this->t('Delete'),
        'regenerate' => $this->t('Regenerate'),
      ],
      '#empty_option' => $this->t('- Select action -'),
    ];

    // Add submit button with proper AJAX configuration
    $form['bulk_operations']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Apply to selected items'),
      '#button_type' => 'primary',
      '#attributes' => [
        'class' => ['use-ajax'],
        'data-dialog-type' => 'modal',
        'data-dialog-options' => json_encode(['width' => 700]),
      ],
      // Instead of defining a custom URL, use the AJAX URL pattern that Drupal expects
      '#ajax' => [
        'callback' => '::submitAjaxCallback',
        // This uses the nojs/ajax pattern that Drupal core expects
        'url' => Url::fromRoute('static_node.bulk_operations_nojs'),
        'options' => [
          'query' => [
            'destination' => \Drupal::destination()->get(),
          ],
        ],
      ],
    ];
    
    // Hidden field to store selected items
    $form['selected_items'] = [
      '#type' => 'hidden',
      '#default_value' => '',
      '#attributes' => ['id' => 'selected-items'],
    ];
    
    return $form;
  }

  /**
   * AJAX callback for the submit button.
   */
  public function submitAjaxCallback(array &$form, FormStateInterface $form_state) {
    // Get the selected files from the hidden field
    $selected_items = $form_state->getValue('selected_items');
    $selected_files = !empty($selected_items) ? explode(',', $selected_items) : [];
    $action = $form_state->getValue('action');
    
    if (empty($selected_files)) {
      $this->messenger->addWarning($this->t('No items selected.'));
      return;
    }
    
    // Store selected files in tempstore based on action
    $tempstore_key = 'static_node_bulk_' . $action;
    $tempstore = $this->tempStoreFactory->get($tempstore_key);
    $tempstore->set($this->currentUser->id(), $selected_files);
    
    // Store destination for return after confirmation
    $destination_tempstore = $this->tempStoreFactory->get('static_node');
    $destination = \Drupal::destination()->get();
    $destination_tempstore->set('bulk_' . $action . '_destination', $destination);
    
    // Determine which form to use
    $route_name = '';
    switch ($action) {
      case 'delete':
        $route_name = 'static_node.files_bulk_delete_confirm';
        break;
        
      case 'regenerate':
        $route_name = 'static_node.files_bulk_generate_confirm';
        break;
    }
    
    // Redirect to the confirmation form - using a modal dialog
    $options = [
      'attributes' => [
        'class' => ['use-ajax'],
        'data-dialog-type' => 'modal',
        'data-dialog-options' => json_encode(['width' => 700]),
      ],
    ];
    
    // Use the Drupal 11 compatible way to redirect in AJAX
    $form_state->setRedirectUrl(Url::fromRoute($route_name, [], $options));
    
    // Return a modal command
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // This is a fallback for non-JS situations
    // In most cases, the AJAX callback will handle the submission
    $selected_items = $form_state->getValue('selected_items');
    $selected_files = !empty($selected_items) ? explode(',', $selected_items) : [];
    $action = $form_state->getValue('action');
    
    if (empty($selected_files)) {
      $this->messenger->addWarning($this->t('No items selected.'));
      return;
    }
    
    // Store selected files in tempstore based on action
    $tempstore_key = 'static_node_bulk_' . $action;
    $tempstore = $this->tempStoreFactory->get($tempstore_key);
    $tempstore->set($this->currentUser->id(), $selected_files);
    
    // Determine which form to use
    $route_name = '';
    switch ($action) {
      case 'delete':
        $route_name = 'static_node.files_bulk_delete_confirm';
        break;
        
      case 'regenerate':
        $route_name = 'static_node.files_bulk_generate_confirm';
        break;
    }
    
    // Redirect to the confirmation form
    $form_state->setRedirectUrl(Url::fromRoute($route_name));
  }
}