<?php

namespace Drupal\static_node\Plugin\Action;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Action\ActionBase;
use Drupal\Core\Form\FormState;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\static_node\Service\StaticNodeHooks;

/**
 * Deletes static version of a node.
 *
 * @Action(
 *   id = "static_node_delete_action",
 *   label = @Translation("Delete static page"),
 *   type = "node",
 *   confirm_form_route_name = "static_node.bulk_delete_confirm"
 * )
 */
class DeleteStaticPageAction extends ActionBase implements ContainerFactoryPluginInterface {
  use StringTranslationTrait;

  /**
   * The static node service.
   *
   * @var \Drupal\static_node\Service\StaticNodeHooks
   */
  protected $staticNodeHooks;

  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a DeleteStaticPageAction object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\static_node\Service\StaticNodeHooks $static_node_hooks
   *   The static node hooks service.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The tempstore factory.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    StaticNodeHooks $static_node_hooks,
    PrivateTempStoreFactory $temp_store_factory,
    AccountInterface $current_user,
    \Drupal\Core\Config\ConfigFactoryInterface $config_factory
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->staticNodeHooks = $static_node_hooks;
    $this->tempStoreFactory = $temp_store_factory;
    $this->currentUser = $current_user;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('static_node.hooks'),
      $container->get('tempstore.private'),
      $container->get('current_user'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function access($node, AccountInterface $account = NULL, $return_as_object = FALSE) {
    // Check basic node access
    $access = $node->access('update', $account, TRUE);
    
    // Check if user has permission to delete static nodes
    $permission_check = $account ? $account->hasPermission('delete static node') : $this->currentUser->hasPermission('delete static node');
    
    // Check if this node type is enabled for static generation (if it's not enabled, it shouldn't be deleted either)
    $enabled_types = $this->configFactory->get('static_node.settings')->get('node_types') ?: [];
    $type_enabled = in_array($node->bundle(), $enabled_types);
    
    // Combine all access checks
    $access = $access->andIf(AccessResult::allowedIf($permission_check && $type_enabled));

    return $return_as_object ? $access : $access->isAllowed();
  }

  /**
   * {@inheritdoc}
   */
  public function execute($node = NULL) {
    $this->executeMultiple([$node]);
    return $this->t('Deleted static version for @title', ['@title' => $node->label()]);
  }

  /**
   * {@inheritdoc}
   */
  public function executeMultiple(array $entities) {
    $info = [];
    /** @var \Drupal\node\NodeInterface[] $entities */
    foreach ($entities as $entity) {
      $info[$entity->id()] = $entity->id();
    }
    
    // Store the node IDs in tempstore for the confirmation form.
    $this->tempStoreFactory
      ->get('static_node_bulk_delete')
      ->set($this->currentUser->id(), $info);
      
    // Store the current destination path in tempstore
    $request = \Drupal::requestStack()->getCurrentRequest();
    if ($request->query->has('destination')) {
      $destination = $request->query->get('destination');
      // Store it for retrieval in the confirmation form
      $this->tempStoreFactory
        ->get('static_node')
        ->set('bulk_delete_destination', $destination);
      
      // Remove it from the request to prevent immediate redirect
      $request->query->remove('destination');
    }
      
    return [];
  }

}
