<?php

namespace Drupal\static_node\Plugin\Action;

use Drupal\Core\Action\ActionBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Access\AccessResult;

/**
 * Generates static pages for nodes.
 *
 * @Action(
 *   id = "static_node_generate_action",
 *   label = @Translation("Generate static page"),
 *   type = "node",
 *   confirm_form_route_name = "static_node.bulk_generate_confirm"
 * )
 */
class GenerateStaticPageAction extends ActionBase implements ContainerFactoryPluginInterface {

  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new GenerateStaticPageAction object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The tempstore factory.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(
    array $configuration, 
    $plugin_id, 
    $plugin_definition, 
    PrivateTempStoreFactory $temp_store_factory, 
    AccountInterface $current_user,
    \Drupal\Core\Config\ConfigFactoryInterface $config_factory
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->tempStoreFactory = $temp_store_factory;
    $this->currentUser = $current_user;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('tempstore.private'),
      $container->get('current_user'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function executeMultiple(array $entities) {
    $info = [];
    /** @var \Drupal\node\NodeInterface[] $entities */
    foreach ($entities as $entity) {
      $info[$entity->id()] = $entity->id();
    }
    
    // Store the node IDs in tempstore for the confirmation form.
    $this->tempStoreFactory
      ->get('static_node_bulk_generate')
      ->set($this->currentUser->id(), $info);
    
    // Store the current destination path in tempstore
    $request = \Drupal::request();
    if ($request->query->has('destination')) {
      $destination = $request->query->get('destination');
      // Store it for retrieval in the confirmation form
      $this->tempStoreFactory
        ->get('static_node')
        ->set('bulk_destination', $destination);
      
      // Remove it from the request to prevent immediate redirect
      $request->query->remove('destination');
    }
      
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function execute($entity = NULL) {
    $this->executeMultiple([$entity]);
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function access($object, AccountInterface $account = NULL, $return_as_object = FALSE) {
    /** @var \Drupal\node\NodeInterface $object */
    $result = $object->access('update', $account, TRUE);
        
    // Check if this node type is enabled for static generation
    $enabled_types = $this->configFactory->get('static_node.settings')->get('node_types') ?: [];
    $type_enabled = in_array($object->bundle(), $enabled_types);
    
    $result = $result->andIf(AccessResult::allowedIf($this->currentUser->hasPermission('generate static node') && $type_enabled));
    
    return $return_as_object ? $result : $result->isAllowed();
  }
}
